package com.qmth.distributed.print.task.templete.execute;

import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.google.common.collect.Lists;
import com.qmth.distributed.print.business.bean.dto.excel.UserImportDto;
import com.qmth.distributed.print.business.entity.SysRole;
import com.qmth.distributed.print.business.enums.RoleTypeEnum;
import com.qmth.distributed.print.business.service.SysRoleService;
import com.qmth.distributed.print.business.util.ExcelUtil;
import com.qmth.distributed.print.business.util.excel.ExcelError;
import com.qmth.distributed.print.common.contant.SystemConstant;
import com.qmth.distributed.print.common.enums.ExceptionResultEnum;
import com.qmth.distributed.print.common.util.Result;
import com.qmth.distributed.print.common.util.ResultUtil;
import com.qmth.distributed.print.task.templete.importData.SyncImportTaskTemplete;
import com.qmth.distributed.print.task.templete.service.TaskLogicService;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.stereotype.Service;
import org.springframework.util.LinkedMultiValueMap;
import org.springframework.web.multipart.MultipartFile;

import javax.annotation.Resource;
import java.io.IOException;
import java.io.Serializable;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.Objects;

/**
 * @Description: 用户导入模版
 * @Param:
 * @return:
 * @Author: wangliang
 * @Date: 2021/3/29
 */
@Service
public class UserImportTempleteService implements SyncImportTaskTemplete, Serializable {
    private final static Logger log = LoggerFactory.getLogger(UserImportTempleteService.class);

    @Resource
    TaskLogicService taskLogicService;

    @Resource
    SysRoleService sysRoleService;

    /**
     * 处理导入
     *
     * @param map
     * @return
     */
    @Override
    public Result importTask(Map<String, Object> map) throws IOException, NoSuchFieldException {
        MultipartFile file = (MultipartFile) map.get(SystemConstant.FILE);
        List<LinkedMultiValueMap<Integer, Object>> finalList = ExcelUtil.excelReader(file.getInputStream(), Lists.newArrayList(UserImportDto.class), (finalExcelList, finalColumnNameList, finalExcelErrorList) -> {
            List<ExcelError> excelErrorTemp = new ArrayList<>();
            for (int i = 0; i < finalExcelList.size(); i++) {
                LinkedMultiValueMap<Integer, Object> excelMap = finalExcelList.get(i);
                List<Object> userImportDtoList = excelMap.get(i);
                for (int y = 0; y < userImportDtoList.size(); y++) {
                    UserImportDto userImportDto = (UserImportDto) userImportDtoList.get(y);
                    excelErrorTemp.addAll(ExcelUtil.checkExcelField(userImportDto, y, i));
                    //查角色名是否存在
                    QueryWrapper<SysRole> sysRoleQueryWrapper = new QueryWrapper<>();
                    sysRoleQueryWrapper.lambda().eq(SysRole::getName, userImportDto.getRole());
                    SysRole sysRole = sysRoleService.getOne(sysRoleQueryWrapper);
                    if (Objects.isNull(sysRole)) {
                        throw ExceptionResultEnum.ERROR.exception(JSONObject.toJSONString(new ExcelError(y + 1, "excel第" + (i + 1) + "个sheet第" + (y + 1) + "行[角色名]不存在")));
                    }
                    if (sysRole.getType() == RoleTypeEnum.SCHOOL_ADMIN && Objects.isNull(userImportDto.getSchoolCode())) {
                        excelErrorTemp.add(new ExcelError(y + 1, "excel第" + (i + 1) + "个sheet第" + (y + 1) + "行[学校代码]必填"));
                    }
                    if (userImportDto.getPassword().length() < 6) {
                        excelErrorTemp.add(new ExcelError(y + 1, "excel第" + (i + 1) + "个sheet第" + (y + 1) + "行[密码]不能小于6位"));
                    } else if (userImportDto.getPassword().length() > 30) {
                        excelErrorTemp.add(new ExcelError(y + 1, "excel第" + (i + 1) + "个sheet第" + (y + 1) + "行[密码]不能大于30位"));
                    }
                    if (userImportDto.getName().length() > 25) {
                        excelErrorTemp.add(new ExcelError(y + 1, "excel第" + (i + 1) + "个sheet第" + (y + 1) + "行[姓名]不能超过25个字符"));
                    }
                    if (userImportDto.getLoginName().length() > 25) {
                        excelErrorTemp.add(new ExcelError(y + 1, "excel第" + (i + 1) + "个sheet第" + (y + 1) + "行[登录名]不能超过25个字符"));
                    }
                    userImportDto.setSysRole(sysRole);
                }
            }
            if (excelErrorTemp.size() > 0) {
                throw ExceptionResultEnum.ERROR.exception(JSONObject.toJSONString(excelErrorTemp));
            }
            return finalExcelList;
        });
        return ResultUtil.ok(taskLogicService.executeUserImportLogic(finalList, map));
    }
}
